/**
 * News Project
 *
 * File:         Main.java
 * RCS:          $Id: Main.java,v 1.18 2008/10/24 21:11:55 drc915 Exp $
 * Description:  Main class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 30, 2006 at 7:31:14 AM
 * Language:     Java
 * Package:      edu.northwestern.news
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2008, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.news;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintWriter;
import java.io.StringReader;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Properties;
import java.util.Random;
import java.util.Set;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.gudy.azureus2.core3.config.COConfigurationListener;
import org.gudy.azureus2.core3.config.impl.ConfigurationDefaults;
import org.gudy.azureus2.core3.config.impl.ConfigurationManager;
import org.gudy.azureus2.core3.internat.MessageText;
import org.gudy.azureus2.core3.logging.LogAlert;
import org.gudy.azureus2.core3.logging.Logger;
import org.gudy.azureus2.core3.util.AERunnable;
import org.gudy.azureus2.core3.util.Debug;
import org.gudy.azureus2.core3.util.IndentWriter;
import org.gudy.azureus2.plugins.PluginException;
import org.gudy.azureus2.plugins.PluginListener;
import org.gudy.azureus2.plugins.UnloadablePlugin;
import org.gudy.azureus2.plugins.clientid.ClientIDManager;
import org.gudy.azureus2.plugins.config.ConfigParameter;
import org.gudy.azureus2.plugins.config.ConfigParameterListener;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadListener;
import org.gudy.azureus2.plugins.download.DownloadManager;
import org.gudy.azureus2.plugins.download.DownloadManagerListener;
import org.gudy.azureus2.plugins.logging.LoggerChannel;
import org.gudy.azureus2.plugins.logging.LoggerChannelListener;
import org.gudy.azureus2.plugins.messaging.generic.GenericMessageRegistration;
import org.gudy.azureus2.plugins.network.ConnectionManager;
import org.gudy.azureus2.plugins.peers.Peer;
import org.gudy.azureus2.plugins.ui.UIException;
import org.gudy.azureus2.plugins.ui.UIInstance;
import org.gudy.azureus2.plugins.ui.UIManager;
import org.gudy.azureus2.plugins.ui.UIManagerListener;
import org.gudy.azureus2.plugins.ui.config.BooleanParameter;
import org.gudy.azureus2.plugins.ui.config.ConfigSection;
import org.gudy.azureus2.plugins.ui.config.IntParameter;
import org.gudy.azureus2.plugins.ui.model.BasicPluginConfigModel;
import org.gudy.azureus2.plugins.ui.model.BasicPluginViewModel;
import org.gudy.azureus2.plugins.update.UpdateCheckInstance;
import org.gudy.azureus2.plugins.update.UpdateManager;
import org.gudy.azureus2.plugins.utils.UTTimer;
import org.gudy.azureus2.plugins.utils.UTTimerEvent;
import org.gudy.azureus2.plugins.utils.UTTimerEventPerformer;
import org.gudy.azureus2.plugins.utils.Utilities;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloader;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderException;
import org.gudy.azureus2.plugins.utils.resourcedownloader.ResourceDownloaderFactory;
import org.gudy.azureus2.pluginsimpl.local.utils.resourcedownloader.ResourceDownloaderFactoryImpl;
import org.gudy.azureus2.ui.swt.Utils;
import org.gudy.azureus2.ui.swt.plugins.UISWTInstance;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener;
import org.gudy.azureus2.ui.swt.shells.MessageSlideShell;


import edu.northwestern.news.anomaly.AnomalyDetector;
import edu.northwestern.news.anomaly.AnomalyLog;
import edu.northwestern.news.anomaly.AnomalyProperties;
import edu.northwestern.news.dht.IDistributedDatabase;
import edu.northwestern.news.dht.azureus.AzureusDistributedDatabase;
import edu.northwestern.news.download.DownloadObserver;
import edu.northwestern.news.stats.StatManager;
import edu.northwestern.news.timer.ITimer;
import edu.northwestern.news.ui.NewsMessageSlideShell;
import edu.northwestern.news.ui.NewsView;
import edu.northwestern.news.MainGeneric;
import edu.northwestern.news.update.NewsUpdater;
import edu.northwestern.news.util.AzPluginInterface;
import edu.northwestern.news.util.PluginInterface;

import org.gudy.azureus2.pluginsimpl.local.torrent.TorrentAttributePluginImpl;

import com.azureus.plugins.aznetmon.main.RSTPacketCountPerformer;
import com.azureus.plugins.aznetmon.main.RSTPacketStats;
/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The Main class launches this plugin.
 */
public class Main extends MainGeneric implements UnloadablePlugin, DownloadListener  {


	
	


	/** whether UI is created yet */
	boolean isCreated = false; 
	static UISWTInstance swtInstance = null;
	  UISWTViewEventListener myView = null;
	  
    static PluginInterface pi_az = null;

    private static GenericMessageRegistration gmr;
    
    private static final String GENERIC_CLUSTER_OBJECT = "Unknown Download (Found through DHT?)";

    DownloadManager dm;
    ConnectionManager cm;
    ClientIDManager cidm;
    Utilities ut;
    BasicPluginViewModel model;
    UpdateCheckInstance uci;
    private static LoggerChannel log;
    private BooleanParameter enableStatsParam;
    private ArrayList<URL> urls = new ArrayList<URL>();
	private UTTimer torrentCheckTimer;
	private boolean USE_NEWS_UPDATE = false;
	private UIManagerListener uiListener;
	BasicPluginConfigModel config_model;
	private LoggerChannelListener logListener;
	private IntParameter reportingTimeoutParam;
	private IntParameter maxPingsParam;
	private Properties props;

	private static final boolean DO_EXPERIMENTS = false;
    /** minimum delay to check for a new torrent, in milliseconds */
    private static final long TORRENT_CHECK_DELAY = 15 * 60 * 1000;
    /** whether to use plugin-specific download-only torrent */
    public static final boolean USE_DO_TORRENT = false;
    
    /** the time (in System.getMillis()) at which the plugin should check for a new torrent to add */
    private static long nextTorrentCheckTime = -1L;

    /** downloader factory */
    protected static ResourceDownloaderFactory rd_factory = ResourceDownloaderFactoryImpl.getSingleton();
	private static AzureusDistributedDatabase ddb;
    
    /** list of download managers */
    HashMap<Download, DownloadObserver> dlObservers;

    private UTTimer timer;
	private IntParameter aveSurveyIntervalParam;
	private AnomalyDetector detector;
	private ITimer anomalyTimer;
	private LoggerChannel myChannel;
	protected boolean canCheckPlugins = false;
	/** timer for RST observations */
	private UTTimer utTimer;
	private UTTimerEvent rstTimerEvent;
	private RSTPacketCountPerformer perf;
	
	private static ArrayList<String> propertiesLocations = new ArrayList<String>();
	private static Main self;
    
    public Main(){
    	super();
    }

	public void loadProperties() {
		props = new Properties();

    	//try {

    	// here's the name of the file

    	boolean found = false;
    	try {
    		for (String location : propertiesLocations){
    			File properties_file = new File(location);
    			// if properties file exists on its own then override any properties file
    			// potentially held within a jar
    			if (properties_file.exists()) {
    				FileInputStream fis = null;

    				try {
    					fis = new FileInputStream(properties_file);

    					props.load(fis);
    					
    					Properties anom = new Properties();
    					String newName = location.replace("news.properties", "anomalies.properties");
        				File anom_props = new File(newName);
        				if (anom_props.exists()){
        					fis = new FileInputStream(anom_props);
        					anom.load(fis);
        					AnomalyProperties.getInstance().loadFromProperties(anom);
        				}
    					found = true;
    					break;
    				} finally {
    					if (fis != null) {
    						fis.close();
    					}
    				}
    			}
    		}
    		if (!found) {
    			// properties file wasn't there, so use the jar
    			ClassLoader classLoader = pi_az.getPluginClassLoader();

    			if (classLoader instanceof URLClassLoader) {
    				URLClassLoader current = (URLClassLoader) classLoader;

    				URL url = current.findResource("news.properties");

    				if (url != null) {
    					props.load(url.openStream());
    				} else {
    					throw (new Exception(
    							"failed to load news.properties from jars"));
    				}
    				
    				url = current.findResource("anomalies.properties");
    				Properties anom = new Properties();
    				if (url != null) {
    					anom.load(url.openStream());
    					AnomalyProperties.getInstance().loadFromProperties(anom);
    				} else {
    					throw (new Exception(
    							"failed to load anomalies.properties from jars"));
    				}
    			}
    		}
    	} catch (Throwable e) {
    		Debug.printStackTrace(e);

    		String msg = "Can't read 'news.properties' for plugin '" +
    		pi_az.getPluginName() + "': file may be missing";

    		Logger.log(new LogAlert(LogAlert.UNREPEATABLE, LogAlert.AT_ERROR,
    				msg));

    		System.out.println(msg);

    		try {
    			throw (new PluginException(msg, e));
    		} catch (PluginException e1) {
    			// TODO Auto-generated catch block
    			e1.printStackTrace();
    		}
    	}


	}

	/* (non-Javadoc)
     * @see org.gudy.azureus2.plugins.Plugin#initialize(org.gudy.azureus2.plugins.PluginInterface)
     */
    public void initialize(org.gudy.azureus2.plugins.PluginInterface arg0) throws PluginException {
    	MainGeneric.setType("Azureus");
    	self = this;
    	dlObservers = new HashMap<Download, DownloadObserver>();
    	pi_az = new AzPluginInterface();
        ((AzPluginInterface)pi_az).setPluginInterface(arg0);
		 log = pi_az.getLogger().getChannel("NEWS"); 

		 arg0.addListener(new PluginListener(){

//			@Override
			public void closedownComplete() {
				// TODO Auto-generated method stub
				
			}

//			@Override
			public void closedownInitiated() {
				try {
					unload();
				} catch (PluginException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			}

//			@Override
			public void initializationComplete() {
				canCheckPlugins  = true;

				log.log("Initializing news version " + pi_az.getPluginVersion());
				
				if (pi_az.getPluginManager().getPluginInterfaceByID("ono")!=null || 
						pi_az.getPluginManager().getPluginInterfaceByID("Ono")!=null){
					if (!NEWSConfiguration.getInstance().isOnoRunning()){
						log.log("Ono present, not recording duplicate stats!");
						NEWSConfiguration.getInstance().setOnoRunning(true);
					}
				}
				
		        pi_az.getUtilities().createThread("Register Usage", new Runnable(){

					public void run() {
						
						StatManager.start();
						
					}});

		        UpdateManager um = pi_az.getUpdateManager();
		     
		        // handle news updates
		        if (USE_NEWS_UPDATE ) checkForUpdate(um);
			}});
        
        propertiesLocations.add(pi_az.getPluginDirectoryName() +
        		java.io.File.separator + "news.properties");
        if (System.getProperties().getProperty("user.home")!=null){
	        propertiesLocations.add(System.getProperties().getProperty("user.home") +
	        		java.io.File.separator + "news.properties");
        }
        
        // initialize properties
        loadProperties(); 
        NEWSConfiguration.getInstance().loadProperties(props);
        
        timer = pi_az.getUtilities().createTimer("Foo");
        
        
        // register listeners for downloads               
        pi_az.getDownloadManager().addListener(new DownloadManagerListener() {
                public void downloadAdded(Download download) {
                    download.addListener(Main.this);
                }

                public void downloadRemoved(Download download) {
                    download.removeListener(Main.this);
                }
            });
        
        dm = pi_az.getDownloadManager();
        if (dm != null){
        	for (Download download : dm.getDownloads()){
        		if (download.getState()==Download.ST_DOWNLOADING || 
        				download.getState() == Download.ST_SEEDING)
        		{
	        		if (!dlObservers.containsKey(download)) {
	                    DownloadObserver dlObs = new DownloadObserver(download);
	                   
	                    // TODO change
//	                    timer.addPeriodicEvent(NEWSConfiguration.getInstance().getDownloadObservationInterval(), 
//	                		   dlObs);
	                }
        		} else download.addListener(this);
        	}
        }

        
        

        // this loads in the internat stuff
        pi_az.getUtilities().getLocaleUtilities()
          .integrateLocalisedMessageBundle("edu.northwestern.news.internat.Messages");

        UIManager ui_manager = pi_az.getUIManager();

        // TODO add GUI options

        config_model =
            ui_manager.createBasicPluginConfigModel(
                    "plugins", "news.name");
        
        
        
        //peers       = config_model.addStringParameter2(
        //        "news.peers", "news.peers", "" );
        
        //final ActionParameter   inject      = config_model.addActionParameter2(
        //        "news.inject.info", "news.inject.button" );
        final NEWSConfiguration nc = NEWSConfiguration.getInstance();
        enableStatsParam     = config_model.addBooleanParameter2(
                "news.enablestats", "news.enablestats", nc.isRecordStats() );
        
        enableStatsParam.addConfigParameterListener(new ConfigParameterListener(){

			public void configParameterChanged(ConfigParameter param) {
				nc.setRecordStats(enableStatsParam.getValue());
				nc.writeProperties(props, propertiesLocations);
				
				
			}});
        
        
        
        
        String maxPingsS = props.getProperty(
		"news.maxPings");
        int maxPings = maxPingsS == null ? 10 : Integer.valueOf(maxPingsS);
        if (maxPings==60) {
        	maxPings = 10;
        	nc.setMaxPings(10);
        	nc.writeProperties(props, propertiesLocations);
        }
        maxPingsParam = config_model.addIntParameter2(
        		"news.maxPings", "news.maxPings", 
        		maxPings);
        maxPingsParam.addConfigParameterListener(new ConfigParameterListener(){

			public void configParameterChanged(ConfigParameter param) {
				nc.setMaxPings(maxPingsParam.getValue());
				nc.writeProperties(props, propertiesLocations);
				
			}});
        
//        aveSurveyIntervalParam     = config_model.addIntParameter2(
//                "news.aveSurveyInterval", "news.aveSurveyInterval", (int)(nc.getMeanSurveyTime()/1000) );
//        
//        aveSurveyIntervalParam.addConfigParameterListener(new ConfigParameterListener(){
//
//			public void configParameterChanged(ConfigParameter param) {
//				long meanSurveyTime = aveSurveyIntervalParam.getValue();
//				double ratio = (meanSurveyTime*1000.0)/nc.getMeanSurveyTime();
//				nc.setMeanSurveyTime(meanSurveyTime*1000);
//				nc.setMinSurveyTime((long)(nc.getMinSurveyTime()*ratio));
//				nc.setMaxSurveyTime((long)(nc.getMaxSurveyTime()*ratio));
//				nc.writeProperties(props, propertiesLocations);
//				
//				
//			}});
        
        String timeoutS = props.getProperty(
		"news.reportingTimeout");
        int timeout = (timeoutS == null ? 60 : Integer.valueOf(timeoutS));
        reportingTimeoutParam = config_model.addIntParameter2(
        		"news.reportingTimeout", "news.reportingTimeout", 
        		timeout);
        reportingTimeoutParam.addConfigParameterListener(new ConfigParameterListener(){

			public void configParameterChanged(ConfigParameter param) {
				nc.setReportingTimeout(reportingTimeoutParam.getValue());
				nc.writeProperties(props, propertiesLocations);
				
			}});

        config_model.addLabelParameter2("news.respect");
        if (pi_az.getUtilities().compareVersions("2.5.0.2", 
        		pi_az.getAzureusVersion())<=0) {
        	config_model.addHyperlinkParameter2("news.teamSig", "http://aqualab.cs.northwestern.edu");
        } else {
        	config_model.addLabelParameter2("news.teamSig");
        }
        
        
//        enableStatsParam.addListener(
//                new ParameterListener()
//                {
//                    public void
//                    parameterChanged(
//                        Parameter   param )
//                    {
//                    	
//                    }
//                });
            
        model = pi_az.getUIManager().createBasicPluginViewModel("news");

        model.getActivity().setVisible(false);
        model.getProgress().setVisible(false);

        // print output to log window
        logListener = new LoggerChannelListener() {
            public void messageLogged(int type, String message) {
                model.getLogArea().appendText(message + "\n");
            }

            public void messageLogged(String str, Throwable error) {
                model.getLogArea().appendText(error.toString() + "\n");
            }
        };
        
        log.addListener(logListener);
        
        uiListener = new UIManagerListener() {
            public void UIAttached(UIInstance instance) {
                if (instance instanceof UISWTInstance) {
                  swtInstance = (UISWTInstance)instance;
                  myView = new NewsView(pi_az);
                  swtInstance.addView(UISWTInstance.VIEW_MAIN, NewsView.VIEWID, myView);
                  
                }
              }

              public void UIDetached(UIInstance instance) {
                if (instance instanceof UISWTInstance) {
                	swtInstance = null;
                  
                }
              }
            };
            
        pi_az.getUIManager().addUIListener(uiListener);

        // create and start experiment manager
        if (DO_EXPERIMENTS ) startExperimentManager();
         

        
        /*try {
			for (Method m : Class.forName("PluginIntefaceImpl").getDeclaredMethods()){
				try {
					Object o = m.invoke(null);
					
				} catch (IllegalArgumentException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (InvocationTargetException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
        
		 newsLogger.getInstance().log("foo", "bar");
		
       UTTimer dumpTimer = pi_az.getUtilities().createTimer("ConfigDumper");
       dumpTimer.addPeriodicEvent(30*1000, new UTTimerEventPerformer(){

		public void perform(UTTimerEvent event) {
			ConfigurationManager cm = ConfigurationManager.getInstance();
		       ByteArrayOutputStream baos = new ByteArrayOutputStream();
		       PrintWriter pw = new PrintWriter(baos);
		       cm.generate(new IndentWriter(pw));
		       pw.flush();
		       String s = baos.toString();
		       System.out.println(s);
		       
		      
		       
		      Set set = ConfigurationDefaults.getInstance().getAllowedParameters();
		      for (Object o : set){
		    	 String temp = (String)o;
		    	 System.out.println(temp + "=" + ConfigurationDefaults.getInstance().getParameter(temp));
		      }
		       
		       //System.out.println(System.getProperties());
			
		}});*/
        
  // TODO implement      

        
        timer.addEvent(10*1000, new UTTimerEventPerformer(){

//			@Override
			public void perform(UTTimerEvent event) {
				if (!NEWSConfiguration.getInstance().isOverride() || NEWSConfiguration.getInstance().isGlobalOverride())
					AnomalyProperties.getInstance().loadFromProperties(MainGeneric.downloadFromURL(
						"http://haleakala.cs.northwestern.edu/news/config/anomalies.properties", 
						60*1000));
				AnomalyProperties.getInstance().writeProperties(propertiesLocations);
				
			}});
        
        timer.addPeriodicEvent(NEWSConfiguration.getInstance().getUpdateCheckFreqSec()*1000, new UTTimerEventPerformer(){

//			@Override
			public void perform(UTTimerEvent event) {
				if (!NEWSConfiguration.getInstance().isOverride() 
						|| NEWSConfiguration.getInstance().isGlobalOverride())
					AnomalyProperties.getInstance().loadFromProperties(MainGeneric.downloadFromURL(
						"http://haleakala.cs.northwestern.edu/news/config/anomalies.properties", 
						60*1000));
				AnomalyProperties.getInstance().writeProperties(propertiesLocations);
				
			}});
        
       
        timer.addPeriodicEvent(60*1000,  new UTTimerEventPerformer(){

//			@Override
			public void perform(UTTimerEvent event) {
				if (!canCheckPlugins) return;
				if (pi_az.getPluginManager().getPluginInterfaceByID("ono")!=null || 
						pi_az.getPluginManager().getPluginInterfaceByID("Ono")!=null){
					if (!NEWSConfiguration.getInstance().isOnoRunning()){
						log.log("Ono present, not recording duplicate stats!");
						NEWSConfiguration.getInstance().setOnoRunning(true);
					}
				}
				
			}});

        anomalyTimer = MainGeneric.createTimer("AnomalyTimer");
       detector = new AnomalyDetector(anomalyTimer);
       
        anomalyTimer.addPeriodicEvent(NEWSConfiguration.getInstance().
        		getSampleInterval()*1000, detector);
        
        AnomalyLog.getInstance(); // fire up the log by getting a reference to it
      //Start the thread to detect RST packets.
        
        Utilities util = pi_az.getUtilities();

        //For now we are getting data for only Windows //ToDo: make it run on multiple OSs
        if( util.isWindows() ){

            utTimer = util.createTimer("RST Packet Monitor",false);
            perf = new RSTPacketCountPerformer(pi_az);

            //start on event within one second.
            utTimer.addEvent(1000,perf);
            //follow-up every 10 minutes.
            rstTimerEvent = utTimer.addPeriodicEvent(RSTPacketStats.INTERVAL,perf);

            //Stats are uploaded through the version server.

        }else{
            //Currently only works with windows
            RSTPacketStats stats = RSTPacketStats.getInstance();
            stats.setStatus("Plug-in currently only works with Windows machines.");
        }
        
    } // end initialize

    public void updateRSTTimer(){
    	if (rstTimerEvent!=null){
    		rstTimerEvent.cancel();
    		rstTimerEvent = utTimer.addPeriodicEvent(RSTPacketStats.INTERVAL,perf);
    	}
    }
    
    /**
     * @param downloader
     * @param data
     */
    protected void writeAndLoadFile(ResourceDownloader downloader,
        InputStream data) {
        File outputFile = new File(pi_az.getPluginDirectoryName() +
                File.separatorChar +
                downloader.getName()
                          .substring(downloader.getName().lastIndexOf('/')));

        if (outputFile.exists()) {
            outputFile.delete();
        }

        FileWriter out;

        try {
            out = new FileWriter(outputFile);

            int c;

            while ((c = data.read()) != -1)
                out.write(c);

            out.flush();
            out.close();

            urls.add(outputFile.toURL());
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * Starts the update checker.
     * @param um The update manager.
     */
    private void checkForUpdate(UpdateManager um) {
        uci = um.createEmptyUpdateCheckInstance(UpdateCheckInstance.UCI_UPDATE,
                "NEWS Updater");
        // TODO add back in when complete
        uci.addUpdatableComponent(new NewsUpdater(this, RD_SIZE_TIMEOUT,
                RD_SIZE_RETRIES, log, pi_az, uci), true);

        uci.start();
    }

    // from here on down, these are download listener actions.
    // can be used for net positioning control by adding/removing "nearby" peers
    public void stateChanged(Download download, int old_state, int new_state) {
        if ((new_state == Download.ST_DOWNLOADING) ||
                (new_state == Download.ST_SEEDING)) {
            if (!dlObservers.containsKey(download)) {
            	DownloadObserver pf = new DownloadObserver(download);
            	dlObservers.put(download, pf);
            	// TODO change
//            	 timer.addPeriodicEvent(NEWSConfiguration.getInstance().getDownloadObservationInterval(), pf);
            }
        } else if ((new_state == Download.ST_QUEUED) ||
                (new_state == Download.ST_STOPPED)) {
            DownloadObserver pf = dlObservers.remove(download);

            if (pf != null) {
                pf.setActive(false);
            }
           
        }
    }

    public void positionChanged(Download download, int oldPosition,
        int newPosition) {
    }

    /* (non-Javadoc)
     * @see java.lang.Object#finalize()
     */
    @Override
    protected void finalize() throws Throwable {
        unload();
    }

    /* (non-Javadoc)
     * @see org.gudy.azureus2.plugins.UnloadablePlugin#unload()
     */
    public void unload() throws PluginException {
    
    	MainGeneric.startShutdown();
    	if (swtInstance!=null) swtInstance.removeViews(UISWTInstance.VIEW_MAIN, NewsView.VIEWID);    	
    	pi_az.getUIManager().removeUIListener(uiListener);
    	model.destroy();
    	config_model.destroy();
    	log.removeListener(logListener);
    	NEWSConfiguration.getInstance().writeProperties(new Properties(), propertiesLocations);
    	AnomalyProperties.getInstance().writeProperties(propertiesLocations);
    	anomalyTimer.destroy();
    
    	
    	try {
        // catch all exceptions so it is sure to unload
    		MainGeneric.getReporter().setExtendedConnection(true);

    		
        	try {onoUnload();} catch (Exception e){
        		e.printStackTrace();
        	}
    		
    	
    	
//    	vivaldiScanner.setActive(false);

//    	if (DEBUG) System.out.println("Stopping News peer manager...");
//    	NewsPeerManager.getInstance().stop();
        
        MainGeneric.getReporter().setExtendedConnection(false);
        MainGeneric.getReporter().disconnect();
        
        if (torrentCheckTimer!=null) torrentCheckTimer.destroy();
        
//        peerFinders.clear();
        
        
    	} catch (Exception e){
    		e.printStackTrace();
    		MainGeneric.getReporter().disconnect();
    	}
    	
    }    

    protected void startExperimentManager() {
    	// TODO implement for news
//		newsExperimentManager oem = new newsExperimentManager(dig);
//		oem.setConnectionManager(getConnectionManager());
//        pi_az.getUtilities().createThread("newsExperimentManager", oem);
	}
    

    
    /**
     * Checks for torrents that this node should connect to for the purpose of
     * getting peers for experiments.
     */
    public static void checkForTorrentToDownload() {
    	
    }
    
    
    public static int getPeerPort(String dest){
        // find the other guy's port via the custom torrent
        for (Download d : pi_az.getDownloadManager().getDownloads()) {
                if (d.getPeerManager() != null) {
                    for (Peer p : d.getPeerManager().getPeers()) {
                        if (p.getIp().contains(dest)) {
                            return p.getTCPListenPort();                            
                        } // end if
                    } // end for peers
                }

                return -1;
        } // end for downloads
        return -1;
    }
    
    public static Properties downloadFromURL(String url, int timeout) {
        InputStream is = null;
        ResourceDownloader dl = null;
        
		Properties details = new Properties();
        try {
			dl = rd_factory.create(new URL(url));
		

			dl = rd_factory.getTimeoutDownloader(dl, timeout);

	        // download the experiment list 
	        is = dl.download();
	        details.load(is);
	        dl.cancel();
        } catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ResourceDownloaderException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
        return details;
    }

	public static boolean isRunning() {
		return pi_az!=null;
	}

	public static PluginInterface getPluginInterface() {
		return pi_az;
	}
	

	public static LoggerChannel getLogger(){
		if (log==null){
			
		}
		return log;
	}

	public static void showView() {
		if (swtInstance!=null) swtInstance.openView(UISWTInstance.VIEW_MAIN, NewsView.VIEWID, null);

	}

	public static IDistributedDatabase getDdb() {
		if (ddb == null) {
			if (pi_az.getDistributedDatabase()!=null) 
				ddb = new AzureusDistributedDatabase(pi_az.getDistributedDatabase());
		} 
		return ddb;
	}
	public static ArrayList<String> getPropertiesLocations() {
		return propertiesLocations;
	}

	public static Main getInstance() {
		
		return self;
	}

}
