/**
 * News Project
 *
 * File:         MainGeneric.java
 * RCS:          $Id: MainGeneric.java,v 1.6 2008/10/26 00:32:19 drc915 Exp $
 * Description:  MainGeneric class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Jan 22, 2007 at 5:11:14 PM
 * Language:     Java
 * Package:      edu.northwestern.news
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.news;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.URL;
import java.net.UnknownHostException;
import java.nio.CharBuffer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Random;
import java.util.Vector;
import java.util.Map.Entry;

import org.gudy.azureus2.plugins.utils.UTTimer;


import edu.northwestern.news.asn.NetworkAdminASNImpl;
import edu.northwestern.news.asn.NetworkAdminASNLookupImpl;
import edu.northwestern.news.dht.IDistributedDatabase;
import edu.northwestern.news.timer.AzureusTimer;
import edu.northwestern.news.timer.ITimer;
import edu.northwestern.news.timer.ITimerEvent;
import edu.northwestern.news.timer.ITimerEventPerformer;
import edu.northwestern.news.NEWSConfiguration;
import edu.northwestern.news.database.DatabaseReporter;
import edu.northwestern.news.database.MySQLReporter;
import edu.northwestern.news.database.NullReporter;
import edu.northwestern.news.database.PhpReporter;
import edu.northwestern.news.Main;
import edu.northwestern.news.util.TraceRouteRunner;
import edu.northwestern.news.stats.StatManager;
import edu.northwestern.news.stats.Statistics;
import edu.northwestern.news.util.PingManager;
import edu.northwestern.news.util.NewsPluginInterface;
import edu.northwestern.news.util.PluginInterface;
import edu.northwestern.news.util.Util;


/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The MainGeneric class ...
 */
public class MainGeneric {
	 public static final int RD_SIZE_RETRIES = 3;
	    public static final int RD_SIZE_TIMEOUT = 10000;

	    protected static boolean isServer;

	    protected Properties props;

	    /** determines whether active network positioning is performed */
	    protected static final boolean DO_NET_POS = true;
	    protected static final boolean TEST_EDGE_RATIO = false;
	    /** time in seconds before azureus stops reporting stats */
		private static final long MAX_SHUTDOWNTIME = 30;
		protected static final boolean DEBUG = false;
	    public static int EXPERIMENT_SLEEP_TIME = 15;
	    public static boolean RUN_EXPERIMENT_MANAGER = true;

	    private ArrayList<URL> urls = new ArrayList<URL>();
	   
	    protected static PluginInterface pi = new NewsPluginInterface();
		private static String type = "NotAzurues";
		private static String publicIp;
		private static Long nextIpUpdate = 0L;
		private static long ipUpdateInterval = 30*60*1000; // 30 minutes
		private static boolean isShuttingDown = false;
		private static ITimer shutdownTimer;
		private static Random rand;
		private static IDistributedDatabase myDdb;




		public static String getPublicIpAddress() {
			if (isAzureus()){
				try {
				if (Main.getPluginInterface().getUtilities().getPublicAddress() != null) {
		            NetworkAdminASNLookupImpl.getInstance().setAddress(
		            		Main.getPluginInterface().getUtilities().getPublicAddress());
					String ip = Main.getPluginInterface().getUtilities().getPublicAddress().toString().substring(1);
		            if (publicIp!=null && !ip.equals(publicIp)){
						Statistics.getInstance().reportIpChange(publicIp, ip, getLocalAddress(ip), Util.currentGMTTime());
					} else if (publicIp==null && ip!=null && ip.length()>0){
						Statistics.getInstance().reportIpChange(ip, ip, getLocalAddress(ip), Util.currentGMTTime());
					}
					publicIp = ip;
		            return ip;
		        } else {
		        	String s = getLocalPublicAddress();
					if (s.startsWith("192.168")) return null;
					publicIp = s;
					nextIpUpdate = System.currentTimeMillis()+ipUpdateInterval;
					return s;
		        }
				} catch (Exception e){
					return null;
				}
			}
			return null;
		}

		public static String getLocalAddress(String ip) throws UnknownHostException {
			String s;
			InetAddress addresses[] = InetAddress.getAllByName(InetAddress.getLocalHost().getHostName());
			// TODO verify that this always picks the "right" IP address
			//s = addresses[addresses.length-1].getHostAddress();
			if (ip!=null){
				for (InetAddress addr : addresses){
					if (addr.getHostAddress().equals(ip)) return addr.getHostAddress();
				}
			}
			
			for (InetAddress addr : addresses){
				if (addr.isSiteLocalAddress()) return addr.getHostAddress();
			}
			return "";
		}
		
		private static String getLocalPublicAddress() throws UnknownHostException {
			String s;
			InetAddress addresses[] = InetAddress.getAllByName(InetAddress.getLocalHost().getHostName());
			// TODO verify that this always picks the "right" IP address
			//s = addresses[addresses.length-1].getHostAddress();
			for (InetAddress addr : addresses){
				if (!addr.isSiteLocalAddress()) return addr.getHostAddress();
			}
			return "";
		}
		
		public static boolean isAzureus(){
			return type.equals("Azureus");
		}

		public static void setType(String string) {
			type = string;
			
		}
		
	
		public static void initialize() {
//			if (isAzureus()) Main.initialize();
//			else MainPlanetLab.initialize();
			
			
		}


		
		public static boolean isShuttingDown(){
			return isShuttingDown;
		}
		
		
		public static void startUp(){
			isShuttingDown = false;
		}

		public static boolean getDhtEnabled() {
			// TODO Auto-generated method stub
			if (isAzureus()) return false;
			else return true;
		}

		public static Random getRandom() {
			if (rand==null){
				rand = new Random();
			}
			return rand;
		}

		public static IDistributedDatabase getDistributedDatabase() {
			if (myDdb == null){
				if (isAzureus()) myDdb = Main.getDdb();
			}
			return myDdb;
		}
		
		public static void setDistributedDatabase(IDistributedDatabase ddb) {
			
			myDdb = ddb;
		}

		public static void createThread(String name, Runnable target) {
			if (isAzureus()) 
				Main.getPluginInterface().getUtilities().createThread(name, target);
			
		}

		public static DatabaseReporter getReporter() {
			// TODO Auto-generated method stub
			if (!NEWSConfiguration.getInstance().isRecordStats()) 
				return NullReporter.getInstance();
			
			if (isAzureus()) return PhpReporter.getInstance();
			else return MySQLReporter.getInstance();
		}
		
		public static ITimer createTimer(String name) {
			if (isAzureus()) {
				return new AzureusTimer(Main.getPluginInterface().getUtilities().createTimer(name));

			
			}
			else return null;
		}
		
		 public void onoUnload(){
		    
		    	
	            
	            if (DEBUG) System.out.println("Stopping ping manager");
		    	PingManager.stop();
		    	if (DEBUG) System.out.println("Stopping traceroutes");
		    	TraceRouteRunner.stop();
		    	
		    	if (DEBUG) System.out.println("Flushing stat manager");
		    	StatManager.flush();
		    	try {
		    		if (DEBUG) System.out.println("Stopping stat manager");
		    		StatManager.stop();
		    	} catch (Exception e){
		    		e.printStackTrace();
		    	}
		    	if (DEBUG) System.out.println("Flushing statistics");
	            Statistics.getInstance().flush();

		        
		        if (DEBUG) System.out.println("Stopping statistics");
		        Statistics.getInstance().stop();
		        
		        	        
		    }
		 
		 
		    public static PluginInterface getPluginInterface() {
		        // TODO Auto-generated method stub
		        if (isAzureus()) return Main.getPluginInterface();
		    	return pi;
		    }
		    
			public static void startShutdown(){
				shutdownTimer = createTimer("NewsShutdownTimer");
				shutdownTimer.addEvent(System.currentTimeMillis()+NEWSConfiguration.getInstance().getReportingTimeout()*1000, 
						new ITimerEventPerformer(){

							public void perform(ITimerEvent event) {
								isShuttingDown = true;
								MainGeneric.getReporter().stopReporting();
								
							}}
				);
			}

			public static ArrayList<String> getPropertiesLocations() {
				if (isAzureus()) return Main.getPropertiesLocations();
				ArrayList<String> locs = new ArrayList<String>();
				locs.add(getPluginInterface().getPluginDirectoryName() +
							java.io.File.separator + "ono.properties");
				return locs;
			}
			
		    public static String getDatabaseName(){
		    	if (isAzureus()) return "ono";
		    	else return "sidestep";
		    	
		    }
		    
		    public static String getDatabaseHost(){
		    	if (isAzureus()) return "merlot.cs.northwestern.edu";
		    	else return "janus.cs.northwestern.edu";
		    	
		    }


			public static Properties downloadFromURL(String url, int timeout) {
				if (isAzureus()) return Main.downloadFromURL( url, timeout);
//				return MainPlanetLab.downloadFromURL(url, timeout);
				else return new Properties();
			}
			
}
