/**
 * NEWS Project
 *
 * File:         AnomalyLog.java
 * RCS:          $Id: AnomalyLog.java,v 1.1 2008/10/24 21:11:55 drc915 Exp $
 * Description:  AnomalyLog class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Oct 24, 2008 at 12:03:13 PM
 * Language:     Java
 * Package:      edu.northwestern.news.anomaly
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2008, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.news.anomaly;

import java.util.LinkedList;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The AnomalyLog class keeps an ordered list of events that have happened lately.
 */
public class AnomalyLog implements AnomalyListener {
	private static final int MAX_ENTRIES = 300;
	private static AnomalyLog self;
	private LinkedList<AlarmRecord> events;
	
	public static AnomalyLog getInstance(){
		if (self == null) self = new AnomalyLog();
		return self;
	}
	
	private AnomalyLog(){
		events = new LinkedList<AlarmRecord>();
		AnomalyDetector.getInstance().addListener(this);
	}
	
	
	public LinkedList<AlarmRecord> getEvents(){
		return (LinkedList<AlarmRecord>) events.clone();
	}

	public void alarmRaised(String region, AnomalyRecord r, int count) {
		if (events.size()>MAX_ENTRIES) events.removeLast();
		AlarmRecord ar = new AlarmRecord(null, r, -1);		
		ar.isAlarm = true;
		r.timeGMT = System.currentTimeMillis();
		events.addFirst(ar);
	}

	public void alertRaised(AnomalyRecord r) {
		AlarmRecord ar = new AlarmRecord(null, r, -1);
		ar.isAlarm = false;
		if (events.size()>MAX_ENTRIES) events.removeLast();
		events.addFirst(ar);
		
	}
}
