/**
 * NEWS Project
 *
 * File:         AnomalyProperties.java
 * RCS:          $Id: AnomalyProperties.java,v 1.5 2008/05/28 20:14:23 drc915 Exp $
 * Description:  AnomalyProperties class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 7, 2008 at 10:24:10 AM
 * Language:     Java
 * Package:      edu.northwestern.news.anomaly
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2008, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.news.anomaly;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;
import java.util.Properties;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The AnomalyProperties class ...
 */
public class AnomalyProperties {

	public static class MovingAverageConfig{
		int windowSize;
		double stdDevThresh;
		int anomalyDirection;
		
		public String toString(){
			return windowSize + ";"+stdDevThresh+";"+anomalyDirection;
		}
	}


	private static AnomalyProperties self;
	
	
	private HashMap<String, MovingAverageConfig> configs;
	
	public MovingAverageConfig getConfig(String name){
		return configs.get(name);
	}

	public AnomalyProperties(){
		configs = new HashMap<String, MovingAverageConfig>();
	}
	
	public static AnomalyProperties getInstance() {
		if (self == null){
			self = new AnomalyProperties();
		} 
		return self;
	}

	public void loadFromProperties(Properties anom) {
		for (Object o : anom.keySet()){
			String name = (String)o;
			String value = (String)anom.get(o);
			if (value!=null){
				String parts[] = value.split("\\;");
				if (parts.length==3){
					MovingAverageConfig mac = new MovingAverageConfig();
					mac.windowSize = Integer.parseInt(parts[0]);
					mac.stdDevThresh = Double.parseDouble(parts[1]);
					mac.anomalyDirection = Integer.parseInt(parts[2]);
					configs.put(name, mac);
				}
			}
		}
		
	}
	
	public void writeProperties(ArrayList<String> locations){
		Properties props = new Properties();
		for (Entry<String, MovingAverageConfig> ent : configs.entrySet()){
			props.put(ent.getKey(), ent.getValue().toString());
		}
		for (String filename : locations){
			filename = filename.replace("news.properties", "anomalies.properties");
			try {
				FileOutputStream fos = new FileOutputStream(filename);
				props.store(fos, "");
				break;
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void changeStdDevs(double newVal){
		synchronized (configs){
			for (MovingAverageConfig conf : configs.values()){
				conf.stdDevThresh=newVal;
			}
		}
	}
	
	public int getNumDataSources(){
		return configs.size();
	}
}
