/**
 * NEWS Project
 *
 * File:         AnomalyRecord.java
 * RCS:          $Id: AnomalyRecord.java,v 1.11 2008/07/20 20:57:15 drc915 Exp $
 * Description:  AnomalyRecord class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      May 7, 2008 at 9:12:04 PM
 * Language:     Java
 * Package:      edu.northwestern.news.anomaly
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2008, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.news.anomaly;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.util.Map.Entry;

import edu.northwestern.news.MainGeneric;
import edu.northwestern.news.util.Util;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The AnomalyRecord class ...
 */
public class AnomalyRecord {
	static int myId = -1;
	public int windowSize;
	public double stdDevThresh;
	public int numAnomalies;
	InetAddress ip = null;
	public int id = -1;
	boolean valid = false;
	
	public long timeGMT = Util.currentGMTTime();
	public int anomalyTypes;
	static final short version = 5;
	
	public AnomalyRecord(){
		try {
			ip = InetAddress.getByName(MainGeneric.getPublicIpAddress());
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (myId==-1 && ip!=null){
			myId = (int)(ip.hashCode()+Util.currentGMTTime());
		}
	}
	
	public void serialize(ByteArrayOutputStream baos) throws IOException {
		baos.write(Util.convertShort(version));
		baos.write(Util.convertInt(windowSize));
		baos.write(Util.convertFloat((float)stdDevThresh));
		baos.write(Util.convertLong(timeGMT));
		baos.write(Util.convertInt(numAnomalies));
		baos.write(Util.convertInt(anomalyTypes));
		if (myId!=-1) baos.write(Util.convertInt(id));
		else baos.write(Util.convertInt(-1));
	}
	
	public void deserialize(ByteBuffer bBuffer) {
		short thisVersion = bBuffer.getShort();
		if (thisVersion!=version) return;
		
		try {
			windowSize = bBuffer.getInt();
			stdDevThresh = bBuffer.getFloat();
			timeGMT = bBuffer.getLong();
			numAnomalies = bBuffer.getInt();
			anomalyTypes = bBuffer.getInt();
			id = bBuffer.getInt();
			
			if (id!=-1) valid = true;
		} catch (Exception e){
			e.printStackTrace();
		}
	}
	
	public static int getMyId(){
		return myId;
	}
}
