/**
 * NuHAL Project
 *
 * File:         DownloadObserver.java
 * RCS:          $Id: DownloadObserver.java,v 1.3 2008/05/14 14:24:14 drc915 Exp $
 * Description:  DownloadObserver class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Nov 6, 2007 at 11:54:59 AM
 * Language:     Java
 * Package:      edu.northwestern.news.download
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.news.download;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.gudy.azureus2.plugins.PluginInterface;
import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.logging.LoggerChannel;
import org.gudy.azureus2.plugins.utils.UTTimerEvent;
import org.gudy.azureus2.plugins.utils.UTTimerEventPerformer;
import org.gudy.azureus2.ui.swt.shells.MessageSlideShell;

import edu.northwestern.news.Main;
import edu.northwestern.news.ui.NewsMessageSlideShell;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The DownloadObserver class collects information about a download.
 */
public class DownloadObserver implements UTTimerEventPerformer {
	
	String name;
	Download d;
	private boolean active;
	
	/** number of milliseconds with availability < 1 to consider torrent dead */
	long deadTorrentTimeout = 1*60*1000;
	
	/** time first suspected dead */
	long deadStartTime = -1;
	
	public DownloadObserver(Download d){
		this.d = d;
		active = true;
	}

	public void perform(UTTimerEvent event) {
		// TODO Auto-generated method stub
		if (!active) event.cancel();
		
		updateDownload();
	}

	private void updateDownload() {
		if (d.getStats().getAvailability()<1) {
			if(deadStartTime < 0) deadStartTime = System.currentTimeMillis();
			if (System.currentTimeMillis()-deadStartTime > deadTorrentTimeout){
				// dead torrent detected!
				PluginInterface pi = Main.getPluginInterface();
				
				Main.getLogger().logAlert(LoggerChannel.LT_INFORMATION, 
						"I think the torrent named " + d.getName() + " is dead...");
			}
		} else {
			deadStartTime = -1;
		}
		
		
		
	}

	public void setActive(boolean b) {
		active = b;		
	}
	
	

}
