/**
 * News Project
 *
 * File:         NewsView.java
 * RCS:          $Id: NewsView.java,v 1.13 2008/10/24 21:11:55 drc915 Exp $
 * Description:  NewsView class (see below)
 * Author:       David Choffnes
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Apr 16, 2007 at 3:33:47 PM
 * Language:     Java
 * Package:      edu.northwestern.news.ui
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2007, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package edu.northwestern.news.ui;

import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import javax.swing.text.DateFormatter;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.gudy.azureus2.core3.internat.MessageText;
import org.gudy.azureus2.core3.ipfilter.BannedIp;
import org.gudy.azureus2.core3.ipfilter.IPFilterListener;
import org.gudy.azureus2.core3.ipfilter.IpFilter;
import org.gudy.azureus2.core3.ipfilter.IpRange;
import org.gudy.azureus2.core3.util.AERunnable;
import org.gudy.azureus2.core3.util.Debug;
import org.gudy.azureus2.plugins.PluginInterface;
import org.gudy.azureus2.plugins.config.ConfigParameter;
import org.gudy.azureus2.plugins.config.ConfigParameterListener;
import org.gudy.azureus2.plugins.utils.LocaleUtilities;
import org.gudy.azureus2.plugins.utils.UTTimer;
import org.gudy.azureus2.plugins.utils.UTTimerEvent;
import org.gudy.azureus2.plugins.utils.UTTimerEventPerformer;
import org.gudy.azureus2.ui.swt.Messages;
import org.gudy.azureus2.ui.swt.Utils;
import org.gudy.azureus2.ui.swt.components.LinkLabel;
import org.gudy.azureus2.ui.swt.config.BooleanParameter;
import org.gudy.azureus2.ui.swt.config.IntParameter;
import org.gudy.azureus2.ui.swt.config.Parameter;
import org.gudy.azureus2.ui.swt.config.ParameterChangeListener;
import org.gudy.azureus2.ui.swt.config.StringListParameter;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEvent;
import org.gudy.azureus2.ui.swt.plugins.UISWTViewEventListener;
import org.gudy.azureus2.ui.swt.views.AbstractIView;

import edu.northwestern.news.NEWSConfiguration;
import edu.northwestern.news.anomaly.AlarmRecord;
import edu.northwestern.news.anomaly.AnomalyDetector;
import edu.northwestern.news.anomaly.AnomalyListener;
import edu.northwestern.news.anomaly.AnomalyLog;
import edu.northwestern.news.anomaly.AnomalyProperties;
import edu.northwestern.news.anomaly.AnomalyRecord;
import edu.northwestern.news.util.Util;

/**
 * @author David Choffnes &lt;drchoffnes@cs.northwestern.edu&gt;
 *
 * The NewsView class ...
 */
public class NewsView extends AbstractIView implements UISWTViewEventListener, AnomalyListener {
  

private class UpdateThread extends Thread {
    boolean bContinue;
    NewsView view;
    
		public UpdateThread(NewsView view) {
			super("News Update Thread");
			this.view = view;
			
		}
    
    public void run() {
      try {
        bContinue = true;
        while(bContinue) {   
          
        	view.refreshActivity();

          
          Thread.sleep(1000);
        }
      } catch(Exception e) {
      	Debug.printStackTrace( e );  
      }
    }
    
    public void stopIt() {
      bContinue = false;
    }
  }

public static class EdgeData{
	public EdgeData(String custName, String ip, Double rtt, Double ratio) {
		this.custName = custName;
		this.ip = ip;
		this.rtt = rtt;
		this.ratio = ratio;
	}
	public String custName;
	public String ip;
	public double rtt;
	public double ratio;
	@Override
	public boolean equals(Object obj) {
		if (obj==null) return false;
		if (obj instanceof EdgeData){
			EdgeData ed = (EdgeData) obj;
			if (ed.ip.equals(ip) && ed.custName.equals(custName)){
				return true;
			}
		}
		return false;
	}
	@Override
	public int hashCode() {
		// TODO Auto-generated method stub
		return ip.hashCode()+custName.hashCode();
	}
	public void copy(EdgeData data) {
		this.rtt = data.rtt;
		this.ratio = data.ratio;
		
	}

}

public static class IpData{
	public String ip;
	public String dl;
	public IpData(String ip, String dl) {
		super();
		this.ip = ip;
		this.dl = dl;
	}
	
	
}

public static class IpStatData {
	public String ip;
	boolean isDHT;
	boolean isDig;
	boolean hasRatio;
	public IpStatData(String ip, boolean isDHT, boolean isDig, boolean hasRatio) {
		super();
		this.ip = ip;
		this.isDHT = isDHT;
		this.isDig = isDig;
		this.hasRatio = hasRatio;
	}
	
	public String getStatus(){
		StringBuffer s = new StringBuffer();
		if (isDHT) s.append("Looked up in DHT");
		if (isDig){
			if (s.length()>0) s.append(" | ");
			s.append("Digging remotely");
		}
		if (hasRatio){
			if (s.length()>0) s.append(" | ");
			s.append("Tracking ratio");
		}
		
		return s.toString();
		
	}
}

public static final String VIEWID = "NEWS";


public static final String EDGE_MAPPINGS = "EdgeMappings";
public static final String NEARBY_PEERS = "NearbyPeers";

/** in seconds */
private static final int PERIODIC_UPDATE_INTERVAL = 5;


private static final String CFG_PREFIX = "news.anomaly.";


private static final int MAX_ALARMS = 50;


private static final int MAX_ALERTS = 50;
  
  Composite panel;
  
  
  boolean activityChanged;
  Table activityTable;
  Table edgeTable;
 

  private boolean isCreated = false;

  UpdateThread updateThread;



	private Map<String, String> allIps;




protected boolean ipsChanged;



private PluginInterface pi;


private UTTimer timer;


boolean listenersAdded = false;


private boolean showEdge = true;


private Table alertTable;


private boolean noChange;

private ArrayList<AnomalyRecord> alerts;

private ArrayList<AlarmRecord> alarms;

  public NewsView( PluginInterface pi ) {
   this.pi = pi;
    init();
  }
  
  private void init() {
	  allIps = new HashMap<String, String>();
	  alerts = new ArrayList<AnomalyRecord>();
	  alarms = new ArrayList<AlarmRecord>();
	  for (AlarmRecord ar : AnomalyLog.getInstance().getEvents()){
		  if (ar.isAlarm){			  
			  alarms.add(ar);
		  } else {
			  alerts.add(ar.record);
		  }
	  }

	    activityChanged = true;
  }
  
  public void initialize(Composite composite) {
    panel = new Composite(composite,SWT.NULL);
    GridLayout layout = new GridLayout();
    layout.numColumns = 2;
    
    panel.setLayout(layout);
    
    timer = pi.getUtilities().createTimer("NewsViewUpdateTimer");
    timer.addPeriodicEvent(PERIODIC_UPDATE_INTERVAL*1000, new UTTimerEventPerformer(){

		public void perform(UTTimerEvent event) {
			periodicUpdate();
			
		}});    
    
    Label label = new Label(panel, SWT.WRAP);
    Messages.setLanguageText(label,"DetailsPane.text.main");
//    Label label2 = new Label(panel, SWT.WRAP);
//    Messages.setLanguageText(label2,"DetailsPane.text.feedback");
    GridData gridData = new GridData(GridData.FILL_HORIZONTAL);

    gridData = new GridData();
	gridData.horizontalSpan = 2;
	gridData.widthHint=400;
	new LinkLabel(panel, gridData, "NEWSView.label.forums",
	"http://forums.aqua-lab.org");
	new LinkLabel(panel, gridData, "NEWSView.label.aqualab",
			"http://aqualab.cs.northwestern.edu");

//	label.setLayoutData(gridData);
    
    
//    final BufferedTruncatedLabel trackerUrlValue = new BufferedTruncatedLabel(panel, SWT.LEFT,70);        
//	
//	  
//
//    trackerUrlValue.setForeground(Colors.blue);
//	  trackerUrlValue.setCursor(Cursors.handCursor);
//	  Messages.setLanguageText(trackerUrlValue.getWidget(), "NEWSView.label.aqualab.tooltip", true);
//    Messages.setLanguageText(trackerUrlValue.getWidget(), "NEWSView.label.aqualab");
//
//	  
//    trackerUrlValue.addMouseListener(new MouseAdapter() {
//			public void mouseDown(MouseEvent event) {
//				if (event.button == 1) {
//					String url = "http://aqualab.cs.northwestern.edu";
//					if (url.startsWith("http://") || url.startsWith("https://")) {
//						int pos = -1;
//						if ((pos = url.indexOf("/announce")) != -1) {
//							url = url.substring(0, pos + 1);
//						}
//						Utils.launch(url);
//					}
//				}
//			}
//		});
    
    
    Group gSettings = new Group(panel, SWT.NULL);
	Messages.setLanguageText(gSettings, CFG_PREFIX + "settings.group");
	gridData = new GridData(GridData.FILL_HORIZONTAL);
	gSettings.setLayoutData(gridData);
//	GridLayout layout = new GridLayout();
//	layout.numColumns = 2;
	gSettings.setLayout(layout);
	
	Label lcrypto = new Label(gSettings, SWT.WRAP);
	Messages.setLanguageText(lcrypto, CFG_PREFIX + "settings.info");
	gridData = new GridData(GridData.FILL_HORIZONTAL);
	gridData.horizontalSpan = 2;
	gridData.widthHint = 200;  // needed for wrap
	lcrypto.setLayoutData(gridData);

//	enableStatsParam     = config_model.addBooleanParameter2(
//            "news.suppressWarnings", "news.suppressWarnings", nc.isRecordStats() );
//    
//    enableStatsParam.addConfigParameterListener(new ConfigParameterListener(){
//
//		public void configParameterChanged(ConfigParameter param) {
//			nc.setSuppressWarnings(enableStatsParam.getValue());
//			nc.writeProperties(props, propertiesLocations);
//			
//			
//		}});
	

	// for overriding default settings
	final BooleanParameter require = new BooleanParameter(gSettings,	
			"news.anomaly.override", 
			"news.anomaly.override");
	gridData = new GridData();
	gridData.horizontalSpan = 2;
	require.setLayoutData(gridData);
	require.setValue(NEWSConfiguration.getInstance().isOverride());

	// for drop-downs
	final Composite cDropDowns = new Composite(gSettings, SWT.NULL);
	gridData = new GridData(GridData.VERTICAL_ALIGN_FILL | 
			GridData.HORIZONTAL_ALIGN_FILL);
	gridData.horizontalSpan = 2;
	cDropDowns.setLayoutData(gridData);
	layout = new GridLayout();
	layout.numColumns = 2;
	layout.marginWidth = 0;
	layout.marginHeight = 0;
	cDropDowns.setLayout(layout);
	cDropDowns.setEnabled(require.isSelected());
	
	
	// for sensitivity drop-down
	LocaleUtilities lu =  pi.getUtilities().getLocaleUtilities();
	String[] sensitivity_types = { 
			lu.getLocalisedMessageText("news.anomaly.sensitivity.very"),
			lu.getLocalisedMessageText("news.anomaly.sensitivity.highly"),
			lu.getLocalisedMessageText("news.anomaly.sensitivity.moderate"),
			lu.getLocalisedMessageText("news.anomaly.sensitivity.somewhat"),
			lu.getLocalisedMessageText("news.anomaly.sensitivity.least"),
			};
	String dropLabels[] = new String[sensitivity_types.length];
	String dropValues[] = new String[sensitivity_types.length];
	double value = 2.0;
	for (int i = 0; i < sensitivity_types.length; i++) {
		dropLabels[i] = sensitivity_types[i];
		dropValues[i] = value+"";
		value+=0.2;
	}
	
	Label lSensitivity = new Label(cDropDowns, SWT.NULL);
	Messages.setLanguageText(lSensitivity, CFG_PREFIX + "sensitivity.label");
	Messages.setLanguageTooltip(lSensitivity, CFG_PREFIX + "sensitivity.label.info");
	final StringListParameter slSensitivity = new StringListParameter(cDropDowns,	
			"news.anomaly.sensitivity.levels", dropValues[2], 
			dropLabels, dropValues);
	slSensitivity.getControl().setEnabled(require.isSelected());
	
	
	// for corroboration drop-down
	String[] corroboration_types = { 
			lu.getLocalisedMessageText("news.anomaly.corroboration.moderate"),
			lu.getLocalisedMessageText("news.anomaly.corroboration.strong"),
			lu.getLocalisedMessageText("news.anomaly.corroboration.vstrong"),
			};
	String corrobDropLabels[] = new String[corroboration_types.length];
	String corrobDropValues[] = new String[corroboration_types.length];
	int corrob = 5;
	for (int i = 0; i < corroboration_types.length; i++) {
		corrobDropLabels[i] = corroboration_types[i];
		corrobDropValues[i] = corrob+"";
		corrob+=5;
	}
	
	Label lCorrob = new Label(cDropDowns, SWT.NULL);
	Messages.setLanguageText(lCorrob, CFG_PREFIX + "corroboration.label");
	Messages.setLanguageTooltip(lCorrob, CFG_PREFIX + "corroboration.label.info");
	final StringListParameter slCorrob = new StringListParameter(cDropDowns,	
			"news.anomaly.corroboration.levels", corrobDropValues[1], 
			corrobDropLabels, corrobDropValues);
	slCorrob.getControl().setEnabled(require.isSelected());
	
	// for number of data sources
	Label lSources = new Label(cDropDowns, SWT.NULL);
	Messages.setLanguageText(lSources, CFG_PREFIX + "sources.label");
	Messages.setLanguageTooltip(lSources, CFG_PREFIX + "sources.label.info");
	final IntParameter ipSources = new IntParameter(cDropDowns, CFG_PREFIX + "sources", 
			2, AnomalyProperties.getInstance().getNumDataSources());
	ipSources.getControl().setEnabled(require.isSelected());
	ipSources.setValue(NEWSConfiguration.getInstance().getAnomCountThreshold());
	
    final BooleanParameter suppressWarnings = new BooleanParameter(gSettings,	
			"news.suppressWarnings", 
			"news.suppressWarnings");
	gridData = new GridData();
	gridData.horizontalSpan = 2;
	suppressWarnings.setLayoutData(gridData);
	suppressWarnings.setValue(NEWSConfiguration.getInstance().isSuppressWarnings());
	
	// group for table
    Group gAnomalyLog = new Group(panel, SWT.NULL);
	Messages.setLanguageText(gAnomalyLog, CFG_PREFIX + "log.group");
	gridData = new GridData(GridData.FILL_HORIZONTAL);
	gridData.heightHint=200;
	gridData.horizontalSpan = 2;
	gAnomalyLog.setLayoutData(gridData);	
//	GridLayout layout = new GridLayout();
//	layout.numColumns = 2;
	gAnomalyLog.setLayout(layout);
	
	// table (shamelessly copied from ConfigSectionIPFilter)
    alertTable = new Table(gAnomalyLog, SWT.SINGLE | SWT.BORDER | SWT.FULL_SELECTION | SWT.VIRTUAL);
    String[] headers = { 
    		"news.anomaly.table.header.time",
    		"news.anomaly.table.header.sources",
    		"news.anomaly.table.header.type",
    		"news.anomaly.table.header.corroboration",
    		"news.anomaly.table.header.region",
    		};
    int[] sizes = { 40, 110, 110, 110, 110};
    int[] aligns = { SWT.LEFT, SWT.CENTER, SWT.CENTER, SWT.CENTER, SWT.CENTER };
    for (int i = 0; i < headers.length; i++) {
      TableColumn tc = new TableColumn(alertTable, aligns[i]);
      tc.setText(headers[i]);
      tc.setWidth(sizes[i]);
      Messages.setLanguageText(tc, headers[i]); //$NON-NLS-1$
    }
    
    
    // TODO implement
    /*
    TableColumn[] columns = table.getColumns();
    columns[0].setData(new Integer(FilterComparator.FIELD_NAME));
    columns[1].setData(new Integer(FilterComparator.FIELD_START_IP));
    columns[2].setData(new Integer(FilterComparator.FIELD_END_IP));
    
    Listener listener = new Listener() {
      public void handleEvent(Event e) {
        TableColumn tc = (TableColumn) e.widget;
        int field = ((Integer) tc.getData()).intValue();
        comparator.setField(field);
        
        if (field == FilterComparator.FIELD_NAME && !bIsCachingDescriptions) {
        	ipFilterManager.cacheAllDescriptions();
        	bIsCachingDescriptions = true;
        }
        ipRanges = getSortedRanges(filter.getRanges());
        table.setItemCount(ipRanges.length);
        table.clearAll();
    	// bug 69398 on Windows
    	table.redraw();
      }
    };
    
    columns[0].addListener(SWT.Selection,listener);
    columns[1].addListener(SWT.Selection,listener);
    columns[2].addListener(SWT.Selection,listener);
    */

    alertTable.setHeaderVisible(true);

    gridData = new GridData(GridData.FILL_BOTH);
    gridData.heightHint = alertTable.getHeaderHeight() * 6;
//		gridData.widthHint = 200;
    alertTable.setLayoutData(gridData);

//    Composite cArea = new Composite(gAnomalyLog, SWT.NULL);
//    layout = new GridLayout();
//    layout.marginHeight = 0;
//    layout.marginWidth = 0;
//    layout.numColumns = 4;
//    cArea.setLayout(layout);
//  	gridData = new GridData(GridData.FILL_HORIZONTAL);
//    cArea.setLayoutData(gridData);

    

    /*
    table.addMouseListener(new MouseAdapter() {
      public void mouseDoubleClick(MouseEvent arg0) {
        TableItem[] selection = table.getSelection();
        if (selection.length == 0)
          return;
        editRange((IpRange) selection[0].getData());
      }
    });*/


//    ipRanges = getSortedRanges(filter.getRanges());
//
//    table.addListener(SWT.SetData, new Listener() {
//			public void handleEvent(Event event) {
//				TableItem item = (TableItem) event.item;
//				int index = table.indexOf(item);
//
//				// seems we can get -1 here (see bug 1219314 )
//
//				if (index < 0 || index >= ipRanges.length) {
//					return;
//				}
//				IpRange range = ipRanges[index];
//				item.setText(0, range.getDescription());
//				item.setText(1, range.getStartIp());
//				item.setText(2, range.getEndIp());
//				item.setData(range);
//			}
//		});
    
    alertTable.setItemCount(alerts.size()+alarms.size());
    alertTable.clearAll();
	// bug 69398 on Windows
	alertTable.redraw();
    
		alertTable.addListener(SWT.Resize, new Listener() {
			public void handleEvent(Event e) {
        resizeTable();
			}
		});

		gAnomalyLog.addListener(SWT.Resize, new Listener() {
			public void handleEvent(Event e) {
        resizeTable();
			}
		});
    
//    
//		filterListener = new IPFilterListener() {
//			public boolean canIPBeBanned(String ip) {
//				return true;
//			}
//		
//			public void IPBanned(BannedIp ip) {
//			}
//
//			public void IPBlockedListChanged(final IpFilter filter) {
//				Utils.execSWTThread(new AERunnable() {
//					public void runSupport() {
//						if (table.isDisposed()) {
//					  	filter.removeListener(filterListener);
//					  	return;
//						}
//		        ipRanges = getSortedRanges(filter.getRanges());
//		        table.setItemCount(ipRanges.length);
//		        table.clearAll();
//		        table.redraw();
//					}
//				});
//				
//			}
//			public boolean
//			canIPBeBlocked(
//				String	ip,
//				byte[]	torrent_hash )
//			{
//				return true;
//			}
//		};
//    filter.addListener(filterListener);
	
	///////////////////////////////////////
	// Actions
	
	require.addChangeListener(new ParameterChangeListener(){

		public void booleanParameterChanging(Parameter p, boolean toValue) {
			NEWSConfiguration.getInstance().setOverride(toValue);
			cDropDowns.setEnabled(toValue);
			slSensitivity.getControl().setEnabled(toValue);
			slCorrob.getControl().setEnabled(toValue);
			ipSources.getControl().setEnabled(toValue);
		}

		public void floatParameterChanging(Parameter owner, double toValue) {
			// TODO Auto-generated method stub
			
		}

		public void intParameterChanging(Parameter p, int toValue) {
			// TODO Auto-generated method stub
			
		}

		public void parameterChanged(Parameter p, boolean caused_internally) {
			// TODO Auto-generated method stub
			
		}

		public void stringParameterChanging(Parameter p, String toValue) {
			// TODO Auto-generated method stub
			
		}});
	
	suppressWarnings.addChangeListener(new ParameterChangeListener(){

		public void booleanParameterChanging(Parameter p, boolean toValue) {
			NEWSConfiguration.getInstance().setSuppressWarnings(toValue);			
		}

		public void floatParameterChanging(Parameter owner, double toValue) {
			// TODO Auto-generated method stub
			
		}

		public void intParameterChanging(Parameter p, int toValue) {
			// TODO Auto-generated method stub
			
		}

		public void parameterChanged(Parameter p, boolean caused_internally) {
			// TODO Auto-generated method stub
			
		}

		public void stringParameterChanging(Parameter p, String toValue) {
			// TODO Auto-generated method stub
			
		}});
	
	slSensitivity.addChangeListener(new ParameterChangeListener(){

		public void booleanParameterChanging(Parameter p, boolean toValue) {
			System.out.println("Changing bool!");
			
		}

		public void floatParameterChanging(Parameter owner, double toValue) {
			System.out.println("Changing float!");
			
		}

		public void intParameterChanging(Parameter p, int toValue) {
			System.out.println("Changing int!");
			
		}

		public void parameterChanged(Parameter p, boolean caused_internally) {
			if (caused_internally){
				AnomalyProperties.getInstance().changeStdDevs(
						Double.parseDouble((String)p.getValueObject()));
			}
			
		}

		public void stringParameterChanging(Parameter p, String toValue) {

			
		}});
	
	slCorrob.addChangeListener(new ParameterChangeListener(){

		public void booleanParameterChanging(Parameter p, boolean toValue) {
			// TODO Auto-generated method stub
			
		}

		public void floatParameterChanging(Parameter owner, double toValue) {
			// TODO Auto-generated method stub
			
		}

		public void intParameterChanging(Parameter p, int toValue) {
			// TODO Auto-generated method stub
			
		}

		public void parameterChanged(Parameter p, boolean caused_internally) {
			if (caused_internally){
				int thresh = Integer.parseInt((String)p.getValueObject());
				NEWSConfiguration.getInstance().setAlarmThreshold(thresh);
			}
			
		}

		public void stringParameterChanging(Parameter p, String toValue) {
			
			
		}});
	
	ipSources.addChangeListener(new ParameterChangeListener(){

		public void booleanParameterChanging(Parameter p, boolean toValue) {
			// TODO Auto-generated method stub
			
		}

		public void floatParameterChanging(Parameter owner, double toValue) {
			// TODO Auto-generated method stub
			
		}

		public void intParameterChanging(Parameter p, int toValue) {
			// TODO Auto-generated method stub
			
		}

		public void parameterChanged(Parameter p, boolean caused_internally) {
			if (caused_internally){
				int thresh = (Integer)p.getValueObject();
				NEWSConfiguration.getInstance().setAnomCountThreshold(thresh);				
			}
			
		}

		public void stringParameterChanging(Parameter p, String toValue) {
			// TODO Auto-generated method stub
			
		}});
    
//    gridData = new GridData(GridData.FILL_HORIZONTAL);
//    gridData.horizontalSpan = 2;
//    trackerUrlValue.setLayoutData(gridData);
//    
//    Composite c = new Composite(panel, SWT.NULL);
//    c.setLayout(new GridLayout());
//    c.setLayoutData(new GridData(SWT.FILL,SWT.FILL,true,true));
//    SashForm sf = new SashForm(c, SWT.BORDER |SWT.HORIZONTAL);
//    sf.setLayout(new GridLayout());
//    sf.setLayoutData(new GridData(SWT.FILL,SWT.FILL,true,true));
//    
//    initialiseActivityGroup(sf);    
//    if (showEdge ) initializeEdgeGroup(sf);
	    

  }
  
  
  
  private void initialiseActivityGroup(SashForm c) {
	  Composite temp = new Composite(c, SWT.NULL);
	  temp.setLayout(new GridLayout());
	  temp.setLayoutData(new GridData(SWT.FILL,SWT.FILL,true,true));

//    Messages.setLanguageText(temp,"NewsView.activity.title");


    Label label = new Label(temp, SWT.WRAP);
    Messages.setLanguageText(label,"NearbyPeers.intro");
    

  }
  
  private void initializeEdgeGroup(SashForm c) {

	  Composite temp = new Composite(c, SWT.NULL);
	  temp.setLayout(new GridLayout());
	  temp.setLayoutData(new GridData(SWT.FILL,SWT.FILL,true,true));
//		Messages.setLanguageText(temp,"NewsView.edge.title");


	    
	    final Label label = new Label(temp, SWT.WRAP);
	    Messages.setLanguageText(label,"EdgeMappings.intro");

    	

	  }

  
  public String getFullTitle() {
    return MessageText.getString("NewsView.title.full");
  }
  
  public Composite getComposite() {
    return panel;
  }

  
  private void refreshActivity() {

  }

private void refreshEdge() {

  }
  
  public EdgeData[] getEdgeDataInArray() {
	  return null;
	
}
  
private double getMinPing(String key, Map<String, Double> edgePings) {
	return -1;

}

  
  public void periodicUpdate() {
    //getPeerIpInformation();
//	  refreshEdge();
//	  refreshActivity();
	  if (!listenersAdded){
		  if (AnomalyDetector.getInstance()!=null){
			  AnomalyDetector.getInstance().addListener(this);
			  listenersAdded=true;
		  }
	  }
	  refresh();
  }
  
  

public String getData() {
    return "DHTView.title.full";
  }

	public boolean eventOccurred(UISWTViewEvent event) {
		switch (event.getType()) {
		case UISWTViewEvent.TYPE_CREATE:
	        if (isCreated){
	        	noChange = false;
	        	refresh();
	          return false;
	        }
	        noChange = false;
	        isCreated  = true;
	        break;
	        
		 case UISWTViewEvent.TYPE_DESTROY:
		        delete();
		        isCreated = false;
		        break;
		        
		        
		 case UISWTViewEvent.TYPE_INITIALIZE:
		        initialize((Composite)event.getData());
		        break;

	      case UISWTViewEvent.TYPE_LANGUAGEUPDATE:
	        //updateLanguage();
	        break;

	      case UISWTViewEvent.TYPE_REFRESH:
	        refresh();
	        break;
	    }

	    return true;
	}


	private void initializeUI(Composite composite) {
		System.out.println("Initializing UI!");
		
	}

	public void delete() {
		timer.destroy();
	    Utils.disposeComposite(panel);
	    
//	    updateThread.stopIt();
//	    if (dht != null) {
//	      dht.getControl().removeListener(controlListener);
//	    }
//	    outGraph.dispose();
//	    inGraph.dispose();
		
	}
    
	
	public void refresh() {
	    if (alertTable == null || alertTable.isDisposed() || noChange)
	      return;
	  
	    try {
	    alertTable.removeAll();
	    
	    alertTable.setItemCount(alerts.size()+alarms.size());
	    } catch (Exception e) {
	       return; // access violation, try again later
	    }
	    
	    noChange = true;
	      
	    int count = 0;
	    TableItem[] items = alertTable.getItems();
	    for (int i = 0; i < items.length; i++) {
	    	if (items[i]==null) items[i] = new TableItem(alertTable, 5);
	    	
	    	if (count<alarms.size()){
	    		items[i].setText(0, DateFormat.getDateTimeInstance().format(
	    				alarms.get(count).record.timeGMT));
	    		items[i].setText(1,alarms.get(count).record.numAnomalies+"");
	    		items[i].setText(2,pi.getUtilities().getLocaleUtilities().getLocalisedMessageText("news.misc.confirmed"));
	    		items[i].setText(3,alarms.get(count).count+"");
	    		items[i].setText(4,alarms.get(count).region);
	    	} else {
	    		int newCount = count-alarms.size();
	    		items[i].setText(0, DateFormat.getDateTimeInstance().format(
	    				Util.fromGMTTime(alerts.get(newCount).timeGMT)));
	    		items[i].setText(1,alerts.get(newCount).numAnomalies+"");
	    		items[i].setText(2,pi.getUtilities().getLocaleUtilities().getLocalisedMessageText("news.misc.suspected"));
	    		items[i].setText(3,"*");
	    		items[i].setText(4,"*");
	    	}
	    	count++;
//	      if (items[i] == null || items[i].isDisposed())
//	        continue;
//	      String tmp = items[i].getText(0);
//	      IpRange range = (IpRange) items[i].getData();
//	      
//	      String	desc = range.getDescription();
//
//	      if (desc != null && !desc.equals(tmp))
//	        items[i].setText(0, desc);
//
//	      tmp = items[i].getText(1);
//	      if (range.getStartIp() != null && !range.getStartIp().equals(tmp))
//	        items[i].setText(1, range.getStartIp());
//
//	      tmp = items[i].getText(2);
//	      if (range.getEndIp() != null && !range.getEndIp().equals(tmp))
//	        items[i].setText(2, range.getEndIp());

	    }
	
	  }
	  
//	  protected IpRange[]
//	  getSortedRanges(
//	  		IpRange[]	ranges )
//	  {
//	  	Arrays.sort(
//	  		ranges,
//			comparator);
//	  	
//	  	return( ranges );
//		
//	  }
	  
	  private void resizeTable() {
		  int iNewWidth = alertTable.getClientArea().width - 
	                    alertTable.getColumn(1).getWidth() - 
	                    alertTable.getColumn(2).getWidth() -
	                    alertTable.getColumn(3).getWidth() -
	                    alertTable.getColumn(4).getWidth()- 20;
	    if (iNewWidth > 50)
	      alertTable.getColumn(0).setWidth(iNewWidth);    
	    
	    
//	    int height = 6;
//	    if (table.getItemCount() > height) height = table.getItemCount();
//	    if (height > 20) height = 20;
//	    table.setSize(table.getClientArea().width, height*table.getItemHeight());
	  }

	
	public void alarmRaised(String region, AnomalyRecord r, int count) {
		if (alarms.size()>MAX_ALARMS) alarms.remove(alarms.size()-1);
		r.timeGMT = System.currentTimeMillis();
		alarms.add(0,new AlarmRecord(region, r, count));
		noChange = false;
	}

	
	public void alertRaised(AnomalyRecord r) {
		if (alerts.size()>MAX_ALERTS) alerts.remove(alerts.size()-1);
		alerts.add(0,r);
		noChange = false;
	}
}


